@@ -143,7 +143,6 @@ cd_cache_get_best_mount_for_drive (GDrive *drive,
     GFile *file;
     int new_rank;
 
-    new_rank = 0;
     v = l->data;
 
     m = g_volume_get_mount (v);
@@ -265,13 +264,15 @@ cd_cache_has_content_type (CdCache *cache, const char *content_type)
 static char *
 cd_cache_local_file_to_archive (const char *filename)
 {
-  char *escaped, *retval, *uri;
+  char *escaped, *escaped2, *retval, *uri;
 
   uri = g_filename_to_uri (filename, NULL, NULL);
   escaped = g_uri_escape_string (uri, NULL, FALSE);
-  g_free (uri);
-  retval = g_strdup_printf ("archive://%s", escaped);
+  escaped2 = g_uri_escape_string (escaped, NULL, FALSE);
   g_free (escaped);
+  g_free (uri);
+  retval = g_strdup_printf ("archive://%s/", escaped2);
+  g_free (escaped2);
 
   return retval;
 }
@@ -810,7 +811,9 @@ totem_cd_detect_type_with_url (const char *device,
       type = MEDIA_TYPE_ERROR;
       /* No error, it's just not usable */
     } else {
-      *mrl = g_strdup (cache->mountpoint);
+      *mrl = g_filename_to_uri (cache->mountpoint, NULL, NULL);
+      if (*mrl == NULL)
+	*mrl = g_strdup (cache->mountpoint);
     }
     break;
   default:
