@@ -80,11 +80,10 @@ totem_pl_parser_write_xspf (TotemPlParser *parser, GtkTreeModel *model,
 			   gpointer user_data, GError **error)
 {
 	GFileOutputStream *stream;
-	int num_entries_total, num_entries, i;
+	int num_entries_total, i;
 	char *buf;
 	gboolean success;
 
-	num_entries = totem_pl_parser_num_entries (parser, model, func, user_data);
 	num_entries_total = gtk_tree_model_iter_n_children (model, NULL);
 
 	stream = g_file_replace (output, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
@@ -101,28 +100,31 @@ totem_pl_parser_write_xspf (TotemPlParser *parser, GtkTreeModel *model,
 
 	for (i = 1; i <= num_entries_total; i++) {
 		GtkTreeIter iter;
-		char *url, *url_escaped, *relative, *title;
+		char *uri, *uri_escaped, *relative, *title;
+		GFile *file;
 		gboolean custom_title;
 
 		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
 			continue;
 
-		func (model, &iter, &url, &title, &custom_title, user_data);
+		func (model, &iter, &uri, &title, &custom_title, user_data);
+		file = g_file_new_for_uri (uri);
 
-		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
-		{
-			g_free (url);
+		if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
+			g_object_unref (file);
+			g_free (uri);
 			g_free (title);
 			continue;
 		}
+		g_object_unref (file);
 
-		relative = totem_pl_parser_relative (output, url);
-		url_escaped = g_markup_escape_text (relative ? relative : url, -1);
+		relative = totem_pl_parser_relative (output, uri);
+		uri_escaped = g_markup_escape_text (relative ? relative : uri, -1);
 		buf = g_strdup_printf ("  <track>\n"
-					"   <location>%s</location>\n", url_escaped);
+					"   <location>%s</location>\n", uri_escaped);
 		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
-		g_free (url);
-		g_free (url_escaped);
+		g_free (uri);
+		g_free (uri_escaped);
 		g_free (relative);
 		g_free (buf);
 		if (success == FALSE) {
@@ -148,7 +150,7 @@ totem_pl_parser_write_xspf (TotemPlParser *parser, GtkTreeModel *model,
 	success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 	g_free (buf);
 
-	g_output_stream_close (G_OUTPUT_STREAM (stream), NULL, NULL);
+	g_object_unref (stream);
 
 	return success;
 }
@@ -158,19 +160,20 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 		xmlNodePtr parent)
 {
 	xmlNodePtr node;
-	xmlChar *title, *url, *image_url, *artist, *album, *duration, *moreinfo;
-	xmlChar *download_url, *id;
+	xmlChar *title, *uri, *image_uri, *artist, *album, *duration, *moreinfo;
+	xmlChar *download_uri, *id;
 	GFile *resolved;
+	char *resolved_uri;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
 	
 	title = NULL;
-	url = NULL;
-	image_url = NULL;
+	uri = NULL;
+	image_uri = NULL;
 	artist = NULL;
 	album = NULL;
 	duration = NULL;
 	moreinfo = NULL;
-	download_url = NULL;
+	download_uri = NULL;
 	id = NULL;
 
 	for (node = parent->children; node != NULL; node = node->next)
@@ -179,11 +182,11 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 			continue;
 
 		if (g_ascii_strcasecmp ((char *)node->name, "location") == 0)
-			url = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+			uri = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 		else if (g_ascii_strcasecmp ((char *)node->name, "title") == 0)
 			title = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 		else if (g_ascii_strcasecmp ((char *)node->name, "image") == 0)
-			image_url = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+			image_uri = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 		/* Last.fm uses creator for the artist */
 		else if (g_ascii_strcasecmp ((char *)node->name, "creator") == 0)
 			artist = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
@@ -197,7 +200,7 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 				if (g_ascii_strcasecmp ((char *) rel, "http://www.last.fm/trackpage") == 0)
 					moreinfo = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 				else if (g_ascii_strcasecmp ((char *) rel, "http://www.last.fm/freeTrackURL") == 0)
-					download_url = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+					download_uri = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 				xmlFree (rel);
 			} else {
 				/* If we don't have a rel="", then it's not a last.fm playlist */
@@ -209,24 +212,24 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 			id = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 	}
 
-	if (url == NULL) {
+	if (uri == NULL) {
 		retval = TOTEM_PL_PARSER_RESULT_ERROR;
 		goto bail;
 	}
 
-	if (base_file != NULL)
-		resolved = g_file_resolve_relative_path (base_file, (const char *) url);
-	else
-		resolved = g_file_new_for_uri ((const char *) url);
-	totem_pl_parser_add_url (parser,
+	resolved_uri = totem_pl_parser_resolve_uri (base_file, (char *) uri);
+	resolved = g_file_new_for_uri (resolved_uri);
+	g_free (resolved_uri);
+
+	totem_pl_parser_add_uri (parser,
 				 TOTEM_PL_PARSER_FIELD_FILE, resolved,
 				 TOTEM_PL_PARSER_FIELD_TITLE, title,
 				 TOTEM_PL_PARSER_FIELD_DURATION_MS, duration,
-				 TOTEM_PL_PARSER_FIELD_IMAGE_URL, image_url,
+				 TOTEM_PL_PARSER_FIELD_IMAGE_URI, image_uri,
 				 TOTEM_PL_PARSER_FIELD_AUTHOR, artist,
 				 TOTEM_PL_PARSER_FIELD_ALBUM, album,
 				 TOTEM_PL_PARSER_FIELD_MOREINFO, moreinfo,
-				 TOTEM_PL_PARSER_FIELD_DOWNLOAD_URL, download_url,
+				 TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI, download_uri,
 				 TOTEM_PL_PARSER_FIELD_ID, id,
 				 NULL);
 	g_object_unref (resolved);
@@ -235,12 +238,14 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 
 bail:
 	SAFE_FREE (title);
-	SAFE_FREE (url);
-	SAFE_FREE (image_url);
+	SAFE_FREE (uri);
+	SAFE_FREE (image_uri);
 	SAFE_FREE (artist);
 	SAFE_FREE (album);
 	SAFE_FREE (duration);
 	SAFE_FREE (moreinfo);
+	SAFE_FREE (download_uri);
+	SAFE_FREE (id);
 
 	return retval;
 }
@@ -288,6 +293,7 @@ TotemPlParserResult
 totem_pl_parser_add_xspf (TotemPlParser *parser,
 			  GFile *file,
 			  GFile *base_file,
+			  TotemPlParseData *parse_data,
 			  gpointer data)
 {
 	xmlDocPtr doc;
