@@ -25,7 +25,7 @@
 
 #include <glib.h>
 
-#include <gtk/gtktreemodel.h>
+#include <gtk/gtk.h>
 #include "totem-pl-parser-features.h"
 #include "totem-pl-parser-builtins.h"
 
@@ -42,16 +42,18 @@ G_BEGIN_DECLS
  * @TOTEM_PL_PARSER_RESULT_UNHANDLED: The playlist could not be handled.
  * @TOTEM_PL_PARSER_RESULT_ERROR: There was an error parsing the playlist.
  * @TOTEM_PL_PARSER_RESULT_SUCCESS: The playlist was parsed successfully.
- * @TOTEM_PL_PARSER_RESULT_IGNORED: The playlist was ignored due to its scheme or MIME type (see totem_pl_parser_add_ignored_scheme() and totem_pl_parser_add_ignored_mimetype()).
+ * @TOTEM_PL_PARSER_RESULT_IGNORED: The playlist was ignored due to its scheme or MIME type (see totem_pl_parser_add_ignored_scheme()
+ * and totem_pl_parser_add_ignored_mimetype()).
+ * @TOTEM_PL_PARSER_RESULT_CANCELLED: Parsing of the playlist was cancelled part-way through.
  *
  * Gives the result of parsing a playlist.
  **/
-typedef enum
-{
+typedef enum {
 	TOTEM_PL_PARSER_RESULT_UNHANDLED,
 	TOTEM_PL_PARSER_RESULT_ERROR,
 	TOTEM_PL_PARSER_RESULT_SUCCESS,
-	TOTEM_PL_PARSER_RESULT_IGNORED
+	TOTEM_PL_PARSER_RESULT_IGNORED,
+	TOTEM_PL_PARSER_RESULT_CANCELLED
 } TotemPlParserResult;
 
 typedef struct TotemPlParser	       TotemPlParser;
@@ -71,11 +73,13 @@ struct TotemPlParser {
 /* Known metadata fields */
 
 /**
- * TOTEM_PL_PARSER_FIELD_URL:
+ * TOTEM_PL_PARSER_FIELD_URI:
  *
- * Metadata field for an entry's URL.
+ * Metadata field for an entry's URI.
+ *
+ * Since: 2.26
  **/
-#define TOTEM_PL_PARSER_FIELD_URL		"url"
+#define TOTEM_PL_PARSER_FIELD_URI		"url"
 /**
  * TOTEM_PL_PARSER_FIELD_GENRE:
  *
@@ -171,7 +175,7 @@ struct TotemPlParser {
 /**
  * TOTEM_PL_PARSER_FIELD_MOREINFO:
  *
- * Metadata field for an entry's "more info" URL.
+ * Metadata field for an entry's "more info" URI.
  **/
 #define TOTEM_PL_PARSER_FIELD_MOREINFO		"moreinfo"
 /**
@@ -211,18 +215,22 @@ struct TotemPlParser {
  **/
 #define TOTEM_PL_PARSER_FIELD_CONTACT		"contact"
 /**
- * TOTEM_PL_PARSER_FIELD_IMAGE_URL:
+ * TOTEM_PL_PARSER_FIELD_IMAGE_URI:
  *
- * Metadata field for an entry's thumbnail image URL.
+ * Metadata field for an entry's thumbnail image URI.
+ *
+ * Since: 2.26
  **/
-#define TOTEM_PL_PARSER_FIELD_IMAGE_URL		"image-url"
+#define TOTEM_PL_PARSER_FIELD_IMAGE_URI		"image-url"
 /**
- * TOTEM_PL_PARSER_FIELD_DOWNLOAD_URL:
+ * TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI:
  *
- * Metadata field for an entry's download URL. Only used if an alternate download
+ * Metadata field for an entry's download URI. Only used if an alternate download
  * location is available for the entry.
+ *
+ * Since: 2.26
  **/
-#define TOTEM_PL_PARSER_FIELD_DOWNLOAD_URL	"download-url"
+#define TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI	"download-url"
 /**
  * TOTEM_PL_PARSER_FIELD_ID:
  *
@@ -275,8 +283,7 @@ struct TotemPlParserClass {
  *
  * The type of playlist a #TotemPlParser will parse.
  **/
-typedef enum
-{
+typedef enum {
 	TOTEM_PL_PARSER_PLS,
 	TOTEM_PL_PARSER_M3U,
 	TOTEM_PL_PARSER_M3U_DOS,
@@ -286,16 +293,15 @@ typedef enum
 
 /**
  * TotemPlParserError:
- * @TOTEM_PL_PARSER_ERROR_VFS_OPEN: Error on opening a file
- * @TOTEM_PL_PARSER_ERROR_VFS_WRITE: Error on writing a file
+ * @TOTEM_PL_PARSER_ERROR_NO_DISC: Error attempting to open a disc device when no disc is present
+ * @TOTEM_PL_PARSER_ERROR_MOUNT_FAILED: An attempted mount operation failed
  *
  * Allows you to differentiate between different
  * errors occurring during file operations in a #TotemPlParser.
  **/
-typedef enum
-{
-	TOTEM_PL_PARSER_ERROR_VFS_OPEN,
-	TOTEM_PL_PARSER_ERROR_VFS_WRITE,
+typedef enum {
+	TOTEM_PL_PARSER_ERROR_NO_DISC,
+	TOTEM_PL_PARSER_ERROR_MOUNT_FAILED
 } TotemPlParserError;
 
 #define TOTEM_PL_PARSER_ERROR (totem_pl_parser_error_quark ())
@@ -348,11 +354,25 @@ void       totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
 						 const char *mimetype);
 
 TotemPlParserResult totem_pl_parser_parse (TotemPlParser *parser,
-					   const char *url, gboolean fallback);
+					   const char *uri, gboolean fallback);
+void totem_pl_parser_parse_async (TotemPlParser *parser, const char *uri,
+				  gboolean fallback, GCancellable *cancellable,
+				  GAsyncReadyCallback callback,
+                                  gpointer user_data);
+TotemPlParserResult totem_pl_parser_parse_finish (TotemPlParser *parser,
+						  GAsyncResult *async_result,
+						  GError **error);
+
 TotemPlParserResult totem_pl_parser_parse_with_base (TotemPlParser *parser,
-						     const char *url,
+						     const char *uri,
 						     const char *base,
 						     gboolean fallback);
+void totem_pl_parser_parse_with_base_async (TotemPlParser *parser,
+					    const char *uri, const char *base,
+					    gboolean fallback,
+					    GCancellable *cancellable,
+					    GAsyncReadyCallback callback,
+                    			    gpointer user_data);
 
 TotemPlParser *totem_pl_parser_new (void);
 
